"""
A class for working with pyygame sprites generated from text
(C) Copyright 2004 James Paige & Hamster Republic Productions

This file is part of Open Leaping Lemmings.
http://HamsterRepublic.com/games/leaping/

Open Leaping Lemmings is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Open Leaping Lemmings is distributed in the hope that it will be fun,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Open Leaping Lemmings in the file license.txt ;
if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

########################################################################

import pygame
from pygame.locals import *

#----------------------------------------------------------------

def render_with_outline(font,string,color=(255,255,255),borderColor=(0,0,0),antialias=1):
   """return an image of the text with an outline. Note that this takes much longer
      to render than a non-outline string, so the resulting surface should not be
      re-generated every game cycle, or anything crazy like that ;)"""
   text_image = font.render(string,antialias,borderColor)
   text_rect = text_image.get_rect()
   image = pygame.Surface((text_rect.width + 2, text_rect.height + 2))
   for pos in ((1,0),(0,1),(2,1),(1,2),(1,1)):
      # draws background left, top, right, bottom, and center in that order
      image.blit(text_image,pos)
   text_image = font.render(string,antialias,color)
   # draws foreground
   image.blit(text_image,(1,1))
   return image

#----------------------------------------------------------------

class TextSprite(pygame.sprite.Sprite):
   """A class to encapsulate the graphical and dimentional properties of a text string"""

   def __init__(self, font, value="text", color=(255,255,255), bgcolor=(0,0,0), group=()):
      pygame.sprite.Sprite.__init__(self,group)
      self.color = color
      self.bgcolor = bgcolor
      self.font = font
      self.text = value

   def __getattr__(self,name):
      if name == 'text':
         return self._text
      elif name == 'image':
         if self._needUpdate: self.update()
         return self._image
      elif name == 'rect':
         if self._needUpdate: self.update()
         return self._rect
      else:
         raise AttributeError("'TextSprite' module has no attribute '%s'" % name)

   def __setattr__(self,name,value):
      if name == 'text':
         self._needUpdate = 1
         self.__dict__['_text'] = value
      else:
         self.__dict__[name] = value

   def update(self):
      self._needUpdate = 0
      self._image = render_with_outline(self.font,self._text,self.color,self.bgcolor)
      try:
         old_pos = self._rect.topleft
      except AttributeError:
         old_pos = (0,0)
      self._rect = self._image.get_rect()
      if old_pos:
         self._rect.topleft = old_pos
